function [JAC]=L0L1reg(jfirst,Xfirst,yfirst,lambda,kappa)
% Determines the Jacobian by L0 + L1 regularization using an iterative
% approach.
% -------------------------------------------------------------------------
% Input
%
% jfirst: an m^2 x 1 vector obtained by applying the vec operator on the 
% transpose of JACfirst, where JACfirst is a first estimate of the
% Jacobian. jfirst is obtained by applying the Matlab program
% firstestimate.m.
%
% Xfirst: an (n-4)pm x m^2 block diagonal matrix, obtained from the data 
% by applying the Matlab program firstestimate.m.
%
% yfirst: a (n-4)pm x 1 vector, obtained from the approximated time 
% derivatives by applying the Matlab program firstestimate.m.
%
% lambda: penalty parameter.
%
% kappa: penalty parameter that determines the ratio between the L0 and L1
% penalty.
% -------------------------------------------------------------------------
% Output
%
% JAC: Jacobian matrix
% -------------------------------------------------------------------------
% Reference
%
% Diana M. Hendrickx, Margriet M.W.B. Hendriks, Paul H.C. Eilers,
% Age K. Smilde and Huub C.J. Hoefsloot
% Reverse engineering of metabolic networks, a critical assessment.
% Mol. Biosyst., 2011, DOI: 10.1039/C0MB00083C
% -------------------------------------------------------------------------

% Author
% Diana M. Hendrickx, 04 January 2011
% -------------------------------------------------------------------------
% Version 
% 7.8.1.347 (R2009a)
% -------------------------------------------------------------------------

jnew = jfirst;
XX = Xfirst'*Xfirst;
XY = Xfirst'*yfirst;
factor = lambda^2;
m = sqrt(size(jnew,1));
a = find(eye(m)==0); % only penalty on non diagonal elements
oonneess = ones(size(jfirst(a)));
v = zeros(size(jnew));
v(a) = oonneess./(1e-6+abs(jnew(a))+kappa*jnew(a).^2);
L = factor*diag(v);
jold = jnew;
jnew = (XX + L)\(XY);
while (ssq(jold./jold - jnew./jold))>0.00001
    v = zeros(size(jnew));
    v(a) = oonneess./(1e-6+abs(jnew(a))+kappa*jnew(a).^2);
    L = factor*diag(v);
    jold = jnew;
    jnew = (XX + L)\(XY);
end

JAC = reshape(jnew,m,m)';

% -------------------------------------------------------------------------
% Copyright Notice
% This is a license to use and modify SOFTWARE & DATA
% produced by THE BIOSYSTEMS DATA ANALYSIS GROUP (BDA Group)
% OF THE UNIVERSITY OF AMSTERDAM
%
% If you use, modify or redistribute the SOFTWARE & DATA and/or
% your source modifications, you agree
%
% (i) to use the SOFTWARE & DATA and/or your source modifications
% solely as part of your research and not in any commercial
% product ;
%
% (ii) that the SOFTWARE & DATA and/or your source modifications
% will not be distributed for profit ;
%
% (iii) all copyright notices and this license note with the
% SOFTWARE & DATA are retained any redistribution of
% the SOFTWARE & DATA, or any portion thereof ;
%
% (iv) to indemnify, hold harmless, and defend the Biosystems
% Data Analysis Group of the University of Amsterdam
% from and against any claims or lawsuits that arise or
% result from the use of the SOFTWARE & DATA or your source
% modifications .
%
% (v) to properly reference the SOFTWARE & DATA when used in
% your research in any publication that may result from
% that research .
%
% Reserved Rights. The Biosystems Data Analysis Group of the
% University of Amsterdam retain title and all ownership
% rights to the SOFTWARE & DATA.
%
% Copyright 2010 Biosystems Data Analysis Group ;
% University of Amsterdam
%


