function [JACfirst,jfirst,Xfirst,yfirst]=firstestimate(X,Y,s)
% Determines an initial estimate of the Jacobian by ridge regression.
% -------------------------------------------------------------------------
% Input
%
% X: a n x m x p data structure of metabolite concentrations, with n the 
% number of time points, m the number of metabolites and p the number of 
% experiments in the data set.
%
% Y: an (n-4)x m x p structure of the fourth order approximations of the 
% time derivatives of the metabolite concentrations X.
% Y is obtained by applying the Matlab program fourthorder.m on X.
%
% s: 1 x m vector of steady state concentrations
% -------------------------------------------------------------------------
% Output
%
% JACfirst: an m x m matrix, which is a first estimate of the Jacobian.
%
% jfirst: an m^2 x 1 vector obtained by applying the vec operator on the 
% transpose of JACfirst.
%
% Xfirst: an (n-4)pm x m^2 block diagonal matrix.
% Xfirst = [Xblock 0 0 0 ...;0 Xblock 0 0 ...;0 0 Xblock 0 ...; ...],
% where Xblock = [X1;X2;...;Xp] and Xi consists of the 3th until the 
% (n-2)th time point of experiment i minus steady state(for i = 1,...,p).
% Xfirst will be input for L0 + L1 regularization.
%
% yfirst: a (n-4)pm x 1 vector obtained by applying the vec operator on 
% Yblock, where Yblock = [Y1;Y2;...;Yp] and Yi consists of the fourth
% order estimates of the time derivatives of the ith experiment (for 
% i = 1,...,p).
% yfirst will be input for L0 + L1 regularization.
% -------------------------------------------------------------------------
% Reference
%
% Diana M. Hendrickx, Margriet M.W.B. Hendriks, Paul H.C. Eilers,
% Age K. Smilde and Huub C.J. Hoefsloot
% Reverse engineering of metabolic networks, a critical assessment.
% Mol. Biosyst., 2011, DOI: 10.1039/C0MB00083C
% -------------------------------------------------------------------------

% Author
% Diana M. Hendrickx, 04 January 2011
% -------------------------------------------------------------------------
% Version 
% 7.8.1.347 (R2009a)
% -------------------------------------------------------------------------

[n,m,p] = size(X);

Xnew = X(3:(n-2),:,:);

for i = 1:p
    Xblock(1+(i-1)*(n-4):i*(n-4),:) = Xnew(:,:,i)-ones(n-4,1)*s;
    Yblock(1+(i-1)*(n-4):i*(n-4),:) = Y(:,:,i);
end

for j = 1:m
    Xfirst(1+(j-1)*(n-4)*p:j*(n-4)*p,1+(j-1)*m:j*m) = Xblock ;
    yfirst(1+(j-1)*(n-4)*p:j*(n-4)*p,1) = Yblock(:,j);
end

gamma = 0.001; %ridge constant
R = ridge(yfirst,Xfirst,gamma,0);
jfirst = R(2:m^2+1,:);
JACfirst = reshape(jfirst,m,m)';

% -------------------------------------------------------------------------
% Copyright Notice
% This is a license to use and modify SOFTWARE & DATA
% produced by THE BIOSYSTEMS DATA ANALYSIS GROUP (BDA Group)
% OF THE UNIVERSITY OF AMSTERDAM
%
% If you use, modify or redistribute the SOFTWARE & DATA and/or
% your source modifications, you agree
%
% (i) to use the SOFTWARE & DATA and/or your source modifications
% solely as part of your research and not in any commercial
% product ;
%
% (ii) that the SOFTWARE & DATA and/or your source modifications
% will not be distributed for profit ;
%
% (iii) all copyright notices and this license note with the
% SOFTWARE & DATA are retained any redistribution of
% the SOFTWARE & DATA, or any portion thereof ;
%
% (iv) to indemnify, hold harmless, and defend the Biosystems
% Data Analysis Group of the University of Amsterdam
% from and against any claims or lawsuits that arise or
% result from the use of the SOFTWARE & DATA or your source
% modifications .
%
% (v) to properly reference the SOFTWARE & DATA when used in
% your research in any publication that may result from
% that research .
%
% Reserved Rights. The Biosystems Data Analysis Group of the
% University of Amsterdam retain title and all ownership
% rights to the SOFTWARE & DATA.
%
% Copyright 2010 Biosystems Data Analysis Group ;
% University of Amsterdam
%

